#!/usr/bin/python
# -*- coding: utf-8 -*-

import nltk
import wikipedia
from dragonfire.nlplib import TopicExtractor
from dragonfire.timex import TimeDetector
import sys
import contextlib
import cStringIO
import re
from random import randint
from nltk.tag import SennaNERTagger
from collections import Counter

class WikipediaAnsweringMachine():

	@staticmethod
	def answer(speech, user_prefix):
		#tokens = nltk.word_tokenize(speech)
		#tagged = nltk.pos_tag(tokens)
		topic_obj = TopicExtractor(speech)
		result = topic_obj.extract()
		try:
			print "~Topic: " + result[0]
		except:
			pass
		speech = speech.upper()
		if "WHEN" in speech or "BIRTHDATE" in speech or "DATE " in speech or " DATE" in speech or "BORN" in speech:
			with nostdout():
				with nostderr():
					try:
						wikipage = wikipedia.page(result[0])
						wikicontent = "".join([i if ord(i) < 128 else ' ' for i in wikipage.content])
						#print TimeDetector.tag(wikicontent)
						count = Counter(TimeDetector.tag(wikicontent))
						return count.most_common()[0][0]
					except:
						return noanswer(user_prefix)
		elif "WHERE" in speech or "LOCATION" in speech or "ADDRESS" in speech or "COUNTRY" in speech or "CITY" in speech or "STREET" in speech:
			with nostdout():
				with nostderr():
					try:
						wikipage = wikipedia.page(result[0])
						wikicontent = "".join([i if ord(i) < 128 else ' ' for i in wikipage.content])
						wikicontent = re.sub(r'\([^)]*\)', '', wikicontent)
						nertagger = SennaNERTagger('/usr/share/senna')
						tagged = nertagger.tag(wikicontent.split())
						for tag in tagged:
							if tag[1] == 'B-LOC':
								return tag[0]
						return noanswer(user_prefix)
					except:
						return noanswer(user_prefix)
		else:
			with nostdout():
				with nostderr():
					try:
						summary = wikipedia.summary(result[0], sentences=1)
						summary = "".join([i if ord(i) < 128 else ' ' for i in summary])
						summary = re.sub(r'\([^)]*\)', '', summary)
						return summary
					except:
						return noanswer(user_prefix)

def noanswer(user_prefix):
	words_dragonfire = {
	0 : "I'm not that smart, " + user_prefix + ".",
	1 : "Please, be simple.",
	2 : "Excuse me? I have an average IQ."
	}
	return words_dragonfire[randint(0,2)]

@contextlib.contextmanager
def nostdout():
	save_stdout = sys.stdout
	sys.stdout = cStringIO.StringIO()
	yield
	sys.stdout = save_stdout

@contextlib.contextmanager
def nostderr():
	save_stderr = sys.stderr
	sys.stderr = cStringIO.StringIO()
	yield
	sys.stderr = save_stderr

if __name__ == "__main__":
	print WikipediaAnsweringMachine.answer("At eight o'clock on Thursday morning Arthur didn't feel very good.")



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import contextlib
import cStringIO

@contextlib.contextmanager
def nostdout():
	save_stdout = sys.stdout
	sys.stdout = cStringIO.StringIO()
	yield
	sys.stdout = save_stdout

@contextlib.contextmanager
def nostderr():
	save_stderr = sys.stderr
	sys.stderr = cStringIO.StringIO()
	yield
	sys.stderr = save_stderr

import nltk
with nostdout():
	with nostderr():
		try:
			nltk.download("names")
			nltk.download("brown")
		except:
			pass
from nltk.corpus import names
from nltk.corpus import brown
import random

class Classifiers():

	@staticmethod
	def gender_features(word):
		return {'last_letter': word[-1]}

	@staticmethod
	def gender(word):
		labeled_names = ([(name, 'male') for name in names.words('male.txt')] + [(name, 'female') for name in names.words('female.txt')])
		random.shuffle(labeled_names)
		featuresets = [(Classifiers.gender_features(n), gender) for (n, gender) in labeled_names]
		train_set, test_set = featuresets[500:], featuresets[:500]
		classifier = nltk.NaiveBayesClassifier.train(train_set)
		return classifier.classify(Classifiers.gender_features(word))


# This is our fast Part of Speech tagger
#############################################################################
brown_train = brown.tagged_sents(categories='news')
regexp_tagger = nltk.RegexpTagger(
	[(r'^-?[0-9]+(.[0-9]+)?$', 'CD'),
	 (r'(-|:|;)$', ':'),
	 (r'\'*$', 'MD'),
	 (r'(The|the|A|a|An|an)$', 'AT'),
	 (r'.*able$', 'JJ'),
	 (r'^[A-Z].*$', 'NNP'),
	 (r'.*ness$', 'NN'),
	 (r'.*ly$', 'RB'),
	 (r'.*s$', 'NNS'),
	 (r'.*ing$', 'VBG'),
	 (r'.*ed$', 'VBD'),
	 (r'.*', 'NN')
])
unigram_tagger = nltk.UnigramTagger(brown_train, backoff=regexp_tagger)
bigram_tagger = nltk.BigramTagger(brown_train, backoff=unigram_tagger)
#############################################################################


# This is our semi-CFG; Extend it according to your own needs
#############################################################################
cfg = {}
cfg["NNP+NNP"] = "NNP"
cfg["NN+NN"] = "NNI"
cfg["NNI+NN"] = "NNI"
cfg["JJ+JJ"] = "JJ"
cfg["JJ+NN"] = "NNI"
#############################################################################

class TopicExtractor(object):

	def __init__(self, sentence):
		self.sentence = sentence

	# Split the sentence into singlw words/tokens
	def tokenize_sentence(self, sentence):
		tokens = nltk.word_tokenize(sentence)
		return tokens

	# Normalize brown corpus' tags ("NN", "NN-PL", "NNS" > "NN")
	def normalize_tags(self, tagged):
		n_tagged = []
		for t in tagged:
			if t[1] == "NP-TL" or t[1] == "NP":
				n_tagged.append((t[0], "NNP"))
				continue
			if t[1].endswith("-TL"):
				n_tagged.append((t[0], t[1][:-3]))
				continue
			if t[1].endswith("S"):
				n_tagged.append((t[0], t[1][:-1]))
				continue
			n_tagged.append((t[0], t[1]))
		return n_tagged

	# Extract the main topics from the sentence
	def extract(self):

		tokens = self.tokenize_sentence(self.sentence)
		tags = self.normalize_tags(bigram_tagger.tag(tokens))

		merge = True
		while merge:
			merge = False
			for x in range(0, len(tags) - 1):
				t1 = tags[x]
				t2 = tags[x + 1]
				key = "%s+%s" % (t1[1], t2[1])
				value = cfg.get(key, '')
				if value:
					merge = True
					tags.pop(x)
					tags.pop(x)
					match = "%s %s" % (t1[0], t2[0])
					pos = value
					tags.insert(x, (match, pos))
					break

		matches = []
		for t in tags:
			if t[1] == "NNP" or t[1] == "NNI":
			#if t[1] == "NNP" or t[1] == "NNI" or t[1] == "NN":
				matches.append(t[0])
		return matches



if __name__ == "__main__":
	print Classifiers.gender("Mehmet")
	print Classifiers.gender("Ayşe")
	print Classifiers.gender("İsmail")
	print Classifiers.gender("Berna")

	sentence = "Do you know the birthdate of Barrack Obama"
	topic_obj = TopicExtractor(sentence)
	result = topic_obj.extract()
	print "This sentence is about: %s" % ", ".join(result)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import subprocess
import time
import os

FNULL = open(os.devnull, 'w')

songRunning = False
class Data:
	def __init__(self, com="", msg="", sp="False"):
		self.command = com
		self.message = msg
		self.speak = sp
	def interact(self,duration):
		try:
			subprocess.Popen(["notify-send","Dragonfire", self.message])
		except:
			pass
		if self.command != "":
			time.sleep(duration)
			try:
				subprocess.Popen(self.command, stdout=FNULL, stderr=FNULL)
			except:
				pass
		#if self.speak == True:
		#	self.say(self.message)
		#else:
	def say(self,message):
		#if songRunning == True:
		#	subprocess.Popen(["rhythmbox-client","--pause"])
		if len(message) < 10000:
			print "Dragonfire: " + message.upper()
		print "_______________________________________________________________\n"
		proc = subprocess.Popen(["festival","--tts"], stdin=subprocess.PIPE, stdout=FNULL, stderr=FNULL)
		message = "".join([i if ord(i) < 128 else ' ' for i in message])
		proc.stdin.write(message)
		proc.stdin.close()
		#proc.wait()
		#if songRunning == True:
		#	subprocess.Popen(["rhythmbox-client","--play"])
	def espeak(self,message):
		subprocess.Popen(["espeak","-v","en-uk-north",message])

if __name__ == "__main__":
	userin = Data([" "]," ")
	userin.say("I have a female voice but I don't have a gender identity. I'm a computer program.")



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
from lxml import etree
from dragonfire.utilities import Data
from dragonfire.nlplib import Classifiers
from dragonfire.yoda import YodaQA
from subprocess import call, Popen
import time
import subprocess
import urllib2
import wikipedia
from random import randint
import getpass
import os
import re
import xml.etree.ElementTree as ET
from pykeyboard import PyKeyboard
import datetime
from apiclient.discovery import build
from apiclient.errors import HttpError
import glob
import speech_recognition as sr
import inspect
#import aiml
import contextlib
import cStringIO
from dragonfire.learn import Teachable
import uuid

DRAGONFIRE_PATH = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
FNULL = open(os.devnull, 'w')
GENDER_PREFIX = {'male': 'Sir', 'female': 'My Lady'}
CONVO_ID = uuid.uuid4()

def command(speech):
	#here = os.path.dirname(os.path.realpath(__file__))
	#os.chdir(os.path.expanduser("~/yodaqa/"))
	#Popen(["./gradlew","web","-q"], stdout=FNULL, stderr=FNULL)
	#os.chdir(here)

	'''
	kernel = aiml.Kernel()
	with nostdout():
		with nostderr():
			kernel.learn(DRAGONFIRE_PATH + "/aiml/learn.aiml")
	'''

	previous_command = ""
	global inactive

	global user_full_name
	global user_prefix

	while(True):

		line = speech.readline()
		if line.startswith("sentence1: ") or line.startswith("<search failed>"):
			com = google_speech_api()
			if com == "\n" or com == " ":
				com = "Enter"
			original_com = com

			if (com == 0):
				#speech_error()
				continue

			com = com.upper()
			print "You: " + com

			if inactive == 1 and "DRAGON FIRE" not in com and "WAKE UP" not in com and com != "HEY":
				continue

			if "DRAGON FIRE" in com or "WAKE UP" in com or com == "HEY":
				tts_kill()
				inactive = 0
				userin = Data([" "]," ")
				words_dragonfire = {
					0 : "Yes, " + user_prefix + ".",
					1 : "Yes. I'm waiting.",
					2 : "What is your orders?"
				}
				userin.say(words_dragonfire[randint(0,2)])
			elif "GO TO SLEEP" in com:
				tts_kill()
				inactive = 1
				userin = Data(["echo"],"Dragonfire deactivated. To reactivate say 'Dragonfire!' or 'Wake Up!'")
				userin.say("I'm going to sleep")
				userin.interact(0)
				previous_command = com
			elif "ENOUGH" in com:
				print "Dragonfire quiets."
				tts_kill()
			elif "WHO AM I" in com or "SAY MY NAME" in com:
				tts_kill()
				userin = Data(["echo"], user_full_name)
				userin.say("Your name is " + user_full_name + "," + user_prefix + ".")
				userin.interact(0)
				previous_command = com
			elif "MY TITLE IS LADY" in com or "I'M A LADY" in com or "I'M A WOMAN" in com or "I'M A GIRL" in com:
				tts_kill()
				user_prefix = "My Lady"
				userin = Data([" "]," ")
				userin.say("Pardon, " + user_prefix + ".")
			elif "MY TITLE IS SIR" in com or "I'M A MAN" in com or "I'M A BOY" in com:
				tts_kill()
				user_prefix = "Sir"
				userin = Data([" "]," ")
				userin.say("Pardon, " + user_prefix + ".")
			elif "WHAT IS YOUR NAME" in com:
				tts_kill()
				userin = Data([" "]," ")
				userin.say("My name is Dragon Fire.")
				previous_command = com
			elif "WHAT IS YOUR GENDER" in com:
				tts_kill()
				userin = Data([" "]," ")
				userin.say("I have a female voice but I don't have a gender identity. I'm a computer program, " + user_prefix + ".")
				previous_command = com
			elif "FILE MANAGER" in com or "OPEN FILES" in com:
				tts_kill()
				userin = Data(["pantheon-files"],"File Manager")
				userin.say("File Manager")
				userin.interact(0)
				previous_command = com
			elif "WEB BROWSER" in com:
				tts_kill()
				userin = Data(["sensible-browser"],"Web Browser")
				userin.say("Web Browser")
				userin.interact(0)
				previous_command = com
			elif "OPEN BLENDER" in com:
				tts_kill()
				userin = Data(["blender"],"Blender")
				userin.say("Blender 3D computer graphics software")
				userin.interact(0)
				previous_command = com
			elif "PHOTO SHOP" in com or "PHOTO EDITOR" in com:
				tts_kill()
				userin = Data(["gimp"],"GIMP")
				userin.say("Photo editor")
				userin.interact(0)
				previous_command = com
			elif "INKSCAPE" in com:
				tts_kill()
				userin = Data(["inkscape"],"Inkscape")
				userin.say("Inkscape")
				userin.interact(0)
				previous_command = com
			elif "VIDEO EDITOR" in com:
				tts_kill()
				userin = Data(["kdenlive"],"Kdenlive")
				userin.say("Video editor")
				userin.interact(0)
				previous_command = com
			elif "OPEN CAMERA" in com:
				tts_kill()
				userin = Data(["snap-photobooth"],"Camera")
				userin.say("Camera")
				userin.interact(0)
				previous_command = com
			elif "OPEN CALENDAR" in com:
				tts_kill()
				userin = Data(["maya-calendar"],"Calendar")
				userin.say("Calendar")
				userin.interact(0)
				previous_command = com
			elif "OPEN CALCULATOR" in com:
				tts_kill()
				userin = Data(["pantheon-calculator"],"Calculator")
				userin.say("Calculator")
				userin.interact(0)
				previous_command = com
			elif "OPEN STEAM" in com:
				tts_kill()
				userin = Data(["steam"],"Steam")
				userin.say("Steam Game Store")
				userin.interact(0)
				previous_command = com
			elif "SOFTWARE CENTER" in com:
				tts_kill()
				userin = Data(["software-center"],"Software Center")
				userin.say("Software Center")
				userin.interact(0)
				previous_command = com
			elif "OFFICE SUITE" in com:
				tts_kill()
				userin = Data(["libreoffice"],"LibreOffice")
				userin.say("Office Suite")
				userin.interact(0)
				previous_command = com
			elif "OPEN WRITER" in com:
				tts_kill()
				userin = Data(["libreoffice","--writer"],"LibreOffice Writer")
				userin.say("Writer")
				userin.interact(0)
				previous_command = com
			elif "OPEN MATH" in com:
				tts_kill()
				userin = Data(["libreoffice","--math"],"LibreOffice Math")
				userin.say("Math")
				userin.interact(0)
				previous_command = com
			elif "OPEN IMPRESS" in com:
				tts_kill()
				userin = Data(["libreoffice","--impress"],"LibreOffice Impress")
				userin.say("Impress")
				userin.interact(0)
				previous_command = com
			elif "OPEN DRAW" in com:
				tts_kill()
				userin = Data(["libreoffice","--draw"],"LibreOffice Draw")
				userin.say("Draw")
				userin.interact(0)
				previous_command = com
			elif com.startswith("KEYBOARD "):
				tts_kill()
				with nostdout():
					with nostderr():
						k = PyKeyboard()
						for character in original_com[9:]:
							k.tap_key(character)
						k.tap_key(" ")
			elif com == "ENTER":
				tts_kill()
				with nostdout():
					with nostderr():
						k = PyKeyboard()
						k.tap_key(k.enter_key)
			elif "SHUT DOWN THE COMPUTER" in com:
				tts_kill()
				userin = Data(["sudo","poweroff"],"Shutting down")
				userin.say("Shutting down")
				userin.interact(3)
				previous_command = com
			elif com == "GOODBYE" or com == "BYE BYE" or com == "SEE YOU LATER":
				tts_kill()
				userin = Data([" "]," ")
				userin.say("Goodbye, " + user_prefix)
				previous_command = com
				julius_proc.terminate()
				try:
					os.system('rm /tmp/' + str(datetime.date.today().year) + '*.[Ww][Aa][Vv]')
				except:
					pass
				sys.exit(1)
			elif "WIKIPEDIA" in com and "SEARCH" in com:
				tts_kill()
				with nostdout():
					with nostderr():
						search_query = com.replace("SEARCH ", "").replace(" SEARCH", "").replace(" IN WIKIPEDIA", "").replace("IN WIKIPEDIA ", "").replace(" ON WIKIPEDIA", "").replace("ON WIKIPEDIA ", "").replace(" USING WIKIPEDIA", "").replace("USING WIKIPEDIA ", "").replace(" WIKIPEDIA", "").replace("WIKIPEDIA ", "")

						userin = Data(["sensible-browser","http://en.wikipedia.org/wiki/"+search_query.lower()],search_query)
						userin.interact(0)
						wikicontent = ""
						try:
							wikipage = wikipedia.page(search_query)
							wikicontent = "".join([i if ord(i) < 128 else ' ' for i in wikipage.content])
							wikicontent = re.sub(r'\([^)]*\)', '', wikicontent)
						except:
							pass
				userin.say(wikicontent)
				previous_command = com
			elif "YOUTUBE" in com and "SEARCH" in com:
				tts_kill()
				with nostdout():
					with nostderr():
						search_query = com.replace("SEARCH ", "").replace(" SEARCH", "").replace(" IN YOUTUBE", "").replace("IN YOUTUBE ", "").replace(" ON YOUTUBE", "").replace("ON YOUTUBE ", "").replace(" USING YOUTUBE", "").replace("USING YOUTUBE ", "").replace(" YOUTUBE", "").replace("YOUTUBE ", "")

						DEVELOPER_KEY = "AIzaSyAcwHj2qzI7KWDUN4RkBTX8Y4lrU78lncA"
						YOUTUBE_API_SERVICE_NAME = "youtube"
						YOUTUBE_API_VERSION = "v3"

						youtube = build(YOUTUBE_API_SERVICE_NAME, YOUTUBE_API_VERSION, developerKey=DEVELOPER_KEY)

						search_response = youtube.search().list(q=search_query, part="id,snippet", maxResults=1).execute()

						videos = []
						channels = []
						playlists = []

						# Add each result to the appropriate list, and then display the lists of
						# matching videos, channels, and playlists.
						for search_result in search_response.get("items", []):
							if search_result["id"]["kind"] == "youtube#video":
								  #videos.append("%s (%s)" % (search_result["snippet"]["title"], search_result["id"]["videoId"]))
								videos.append(search_result["snippet"]["title"])
								videos.append(search_result["id"]["videoId"])
							elif search_result["id"]["kind"] == "youtube#channel":
								channels.append("%s (%s)" % (search_result["snippet"]["title"], search_result["id"]["channelId"]))
							elif search_result["id"]["kind"] == "youtube#playlist":
								playlists.append("%s (%s)" % (search_result["snippet"]["title"], search_result["id"]["playlistId"]))

						if len(videos) > 1:
							youtube_title = videos[0]
							youtube_url = "https://www.youtube.com/watch?v=%s" % (videos[1])
							userin = Data(["sensible-browser",youtube_url],youtube_title)
							youtube_title = "".join([i if ord(i) < 128 else ' ' for i in youtube_title])
						else:
							youtube_title = "No video found, " + user_prefix + "."
							userin = Data("",youtube_title)
						k = PyKeyboard()
						k.tap_key('space')
				userin.say(youtube_title)
				userin.interact(0)
				time.sleep(3)
				k.tap_key(k.tab_key)
				k.tap_key(k.tab_key)
				k.tap_key(k.tab_key)
				k.tap_key(k.tab_key)
				k.tap_key('f')
			else:
				tts_kill()
				#dragonfire_respond = kernel.respond(com)
				teachable_respond = Teachable.respond("http://teach.dragon.computer/", CONVO_ID, original_com)
				userin = Data([" "]," ")
				if teachable_respond and "WHAT" not in teachable_respond and "WHERE" not in teachable_respond and "WHO" not in teachable_respond and "WHEN" not in teachable_respond and "WHICH" not in teachable_respond and "HOW" not in teachable_respond:
					userin.say(teachable_respond)
				else:
					userin.say("I need to do a brief research on the internet. It may take up to 3 minutes, so please be patient.")
					userin.say(YodaQA.answer("http://qa.ailao.eu", original_com, user_prefix))
				previous_command = com

			#newest = max(glob.iglob('/tmp/' + str(datetime.date.today().year) + '*.[Ww][Aa][Vv]'), key=os.path.getctime)
			#print newest


def tts_kill():
	call(["pkill", "audsp"], stdout=FNULL, stderr=FNULL)
	call(["pkill", "aplay"], stdout=FNULL, stderr=FNULL)

def dragon_greet():
	tts_kill()
	print "_______________________________________________________________\n"
	time = datetime.datetime.now().time()

	global user_full_name
	global user_prefix

	user_full_name = os.popen("getent passwd $LOGNAME | cut -d: -f5 | cut -d, -f1").read()
	user_full_name = user_full_name[:-1]
	user_prefix = GENDER_PREFIX[Classifiers.gender(user_full_name.split(' ', 1)[0])]

	if time < datetime.time(12):
		userin = Data(["echo"],"To activate say 'Dragonfire!' or 'Wake Up!'")
		userin.say("Good morning " + user_prefix)
		userin.interact(0)
	elif datetime.time(12) < time  and time < datetime.time(18):
		userin = Data(["echo"],"To activate say 'Dragonfire!' or 'Wake Up!'")
		userin.say("Good afternoon " + user_prefix)
		userin.interact(0)
	else:
		userin = Data(["echo"],"To activate say 'Dragonfire!' or 'Wake Up!'")
		userin.say("Good evening " + user_prefix)
		userin.interact(0)

def google_speech_api():

	newest_recording = max(glob.iglob('/tmp/' + str(datetime.date.today().year) + '*.[Ww][Aa][Vv]'), key=os.path.getctime)

	r = sr.Recognizer()
	with sr.WavFile(newest_recording) as source:            # use "test.wav" as the audio source
		audio = r.record(source)                        # extract audio data from the file
	try:
		return r.recognize_google(audio)   # recognize speech using Google Speech Recognition
	except:
		pass
	return 0

def speech_error():
	tts_kill()
	userin = Data(["echo"],"An error occurred")
	userin.say("I couldn't understand, please repeat again")
	userin.interact(0)

@contextlib.contextmanager
def nostdout():
	save_stdout = sys.stdout
	sys.stdout = cStringIO.StringIO()
	yield
	sys.stdout = save_stdout

@contextlib.contextmanager
def nostderr():
	save_stderr = sys.stderr
	sys.stderr = cStringIO.StringIO()
	yield
	sys.stderr = save_stderr

def initiate():
	try:
		global inactive
		global julius_proc
		inactive = 1
		dragon_greet()
		# padsp julius -input mic -C julian.jconf | ./getcommand.py
		julius_proc = subprocess.Popen(["padsp", "julius", "-input", "mic", "-C", DRAGONFIRE_PATH + "/julian.jconf"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
		speech = julius_proc.stdout
		command(speech)
		#command(sys.stdin)
	except KeyboardInterrupt:
		julius_proc.terminate()
		with nostdout():
			with nostderr():
				try:
					os.system('rm /tmp/' + str(datetime.date.today().year) + '*.[Ww][Aa][Vv]')
				except:
					pass
		sys.exit(1)

if __name__ == '__main__':
	try:
		inactive = 1
		dragon_greet()
		# padsp julius -input mic -C julian.jconf | ./getcommand.py
		sys.stdin = subprocess.Popen(["padsp", "julius", "-input", "mic", "-C", DRAGONFIRE_PATH + "julian.jconf"], stdout=subprocess.PIPE).stdout
		command(sys.stdin)
	except KeyboardInterrupt:
		sys.exit(1)



import json
import urllib
import urllib2
import time
import sys
import contextlib
import cStringIO
from random import randint
import uuid

class Teachable():

	@staticmethod
	def respond(address, convo_id, query):
		url = address + '/chatbot/conversation_start.php'
		#url = 'http://localhost:4567/q'
		post_params = {
			'say' : query,
			'convo_id' : convo_id
		}

		params = urllib.urlencode(post_params)
		response = urllib2.urlopen(url, params)
		json_response = json.loads(response.read())

		return json_response['botsay']


def noanswer(user_prefix):
	words_dragonfire = {
	0 : "I'm not that smart, " + user_prefix + ".",
	1 : "Please, be simple.",
	2 : "Excuse me? I have an average IQ."
	}
	return words_dragonfire[randint(0,2)]

@contextlib.contextmanager
def nostdout():
	save_stdout = sys.stdout
	sys.stdout = cStringIO.StringIO()
	yield
	sys.stdout = save_stdout

@contextlib.contextmanager
def nostderr():
	save_stderr = sys.stderr
	sys.stderr = cStringIO.StringIO()
	yield
	sys.stderr = save_stderr

if __name__ == "__main__":
	convo_id = uuid.uuid4()
	print Teachable.respond("http://teach.dragon.computer/", convo_id, "Learn the sun is hot")
	print Teachable.respond("http://teach.dragon.computer/", convo_id, "What is the sun")



# Code for tagging temporal expressions in text
# For details of the TIMEX format, see http://timex2.mitre.org/

import re
import string
import os
import sys

# Requires eGenix.com mx Base Distribution
# http://www.egenix.com/products/python/mxBase/
try:
	from mx.DateTime import *
except ImportError:
	print """
Requires eGenix.com mx Base Distribution
http://www.egenix.com/products/python/mxBase/"""

# Predefined strings.
numbers = "(^a(?=\s)|one|two|three|four|five|six|seven|eight|nine|ten| \
		  eleven|twelve|thirteen|fourteen|fifteen|sixteen|seventeen| \
		  eighteen|nineteen|twenty|thirty|forty|fifty|sixty|seventy|eighty| \
		  ninety|hundred|thousand)"
day = "(monday|tuesday|wednesday|thursday|friday|saturday|sunday)"
week_day = "(monday|tuesday|wednesday|thursday|friday|saturday|sunday)"
month = "(january|february|march|april|may|june|july|august|september| \
		  october|november|december)"
dmy = "(year|day|week|month)"
rel_day = "(today|yesterday|tomorrow|tonight|tonite)"
exp1 = "(before|after|earlier|later|ago)"
exp2 = "(this|next|last)"
iso = "\d+[/-]\d+[/-]\d+ \d+:\d+:\d+\.\d+"
year = "((?<=\s)\d{4}|^\d{4})"
regxp1 = "((\d+|(" + numbers + "[-\s]?)+) " + dmy + "s? " + exp1 + ")"
regxp2 = "(" + exp2 + " (" + dmy + "|" + week_day + "|" + month + "))"

reg1 = re.compile(regxp1, re.IGNORECASE)
reg2 = re.compile(regxp2, re.IGNORECASE)
reg3 = re.compile(rel_day, re.IGNORECASE)
reg4 = re.compile(iso)
reg5 = re.compile(year)

class TimeDetector():

	@staticmethod
	def tag(text):

		# Initialization
		timex_found = []

		# re.findall() finds all the substring matches, keep only the full
		# matching string. Captures expressions such as 'number of days' ago, etc.
		found = reg1.findall(text)
		found = [a[0] for a in found if len(a) > 1]
		for timex in found:
			timex_found.append(timex)

		# Variations of this thursday, next year, etc
		found = reg2.findall(text)
		found = [a[0] for a in found if len(a) > 1]
		for timex in found:
			timex_found.append(timex)

		# today, tomorrow, etc
		found = reg3.findall(text)
		for timex in found:
			timex_found.append(timex)

		# ISO
		found = reg4.findall(text)
		for timex in found:
			timex_found.append(timex)

		# Year
		found = reg5.findall(text)
		for timex in found:
			timex_found.append(timex)

		# Tag only temporal expressions which haven't been tagged.
		for timex in timex_found:
			text = re.sub(timex + '(?!</TIMEX2>)', '<TIMEX2>' + timex + '</TIMEX2>', text)

		return timex_found

# Hash function for week days to simplify the grounding task.
# [Mon..Sun] -> [0..6]
hashweekdays = {
	'Monday': 0,
	'Tuesday': 1,
	'Wednesday': 2,
	'Thursday': 3,
	'Friday': 4,
	'Saturday': 5,
	'Sunday': 6}

# Hash function for months to simplify the grounding task.
# [Jan..Dec] -> [1..12]
hashmonths = {
	'January': 1,
	'February': 2,
	'March': 3,
	'April': 4,
	'May': 5,
	'June': 6,
	'July': 7,
	'August': 8,
	'September': 9,
	'October': 10,
	'November': 11,
	'December': 12}

# Hash number in words into the corresponding integer value
def hashnum(number):
	if re.match(r'one|^a\b', number, re.IGNORECASE):
		return 1
	if re.match(r'two', number, re.IGNORECASE):
		return 2
	if re.match(r'three', number, re.IGNORECASE):
		return 3
	if re.match(r'four', number, re.IGNORECASE):
		return 4
	if re.match(r'five', number, re.IGNORECASE):
		return 5
	if re.match(r'six', number, re.IGNORECASE):
		return 6
	if re.match(r'seven', number, re.IGNORECASE):
		return 7
	if re.match(r'eight', number, re.IGNORECASE):
		return 8
	if re.match(r'nine', number, re.IGNORECASE):
		return 9
	if re.match(r'ten', number, re.IGNORECASE):
		return 10
	if re.match(r'eleven', number, re.IGNORECASE):
		return 11
	if re.match(r'twelve', number, re.IGNORECASE):
		return 12
	if re.match(r'thirteen', number, re.IGNORECASE):
		return 13
	if re.match(r'fourteen', number, re.IGNORECASE):
		return 14
	if re.match(r'fifteen', number, re.IGNORECASE):
		return 15
	if re.match(r'sixteen', number, re.IGNORECASE):
		return 16
	if re.match(r'seventeen', number, re.IGNORECASE):
		return 17
	if re.match(r'eighteen', number, re.IGNORECASE):
		return 18
	if re.match(r'nineteen', number, re.IGNORECASE):
		return 19
	if re.match(r'twenty', number, re.IGNORECASE):
		return 20
	if re.match(r'thirty', number, re.IGNORECASE):
		return 30
	if re.match(r'forty', number, re.IGNORECASE):
		return 40
	if re.match(r'fifty', number, re.IGNORECASE):
		return 50
	if re.match(r'sixty', number, re.IGNORECASE):
		return 60
	if re.match(r'seventy', number, re.IGNORECASE):
		return 70
	if re.match(r'eighty', number, re.IGNORECASE):
		return 80
	if re.match(r'ninety', number, re.IGNORECASE):
		return 90
	if re.match(r'hundred', number, re.IGNORECASE):
		return 100
	if re.match(r'thousand', number, re.IGNORECASE):
	  return 1000

# Given a timex_tagged_text and a Date object set to base_date,
# returns timex_grounded_text
def ground(tagged_text, base_date):

	# Find all identified timex and put them into a list
	timex_regex = re.compile(r'<TIMEX2>.*?</TIMEX2>', re.DOTALL)
	timex_found = timex_regex.findall(tagged_text)
	timex_found = map(lambda timex:re.sub(r'</?TIMEX2.*?>', '', timex), \
				timex_found)

	# Calculate the new date accordingly
	for timex in timex_found:
		timex_val = 'UNKNOWN' # Default value

		timex_ori = timex   # Backup original timex for later substitution

		# If numbers are given in words, hash them into corresponding numbers.
		# eg. twenty five days ago --> 25 days ago
		if re.search(numbers, timex, re.IGNORECASE):
			split_timex = re.split(r'\s(?=days?|months?|years?|weeks?)', \
															  timex, re.IGNORECASE)
			value = split_timex[0]
			unit = split_timex[1]
			num_list = map(lambda s:hashnum(s),re.findall(numbers + '+', \
										  value, re.IGNORECASE))
			timex = `sum(num_list)` + ' ' + unit

		# If timex matches ISO format, remove 'time' and reorder 'date'
		if re.match(r'\d+[/-]\d+[/-]\d+ \d+:\d+:\d+\.\d+', timex):
			dmy = re.split(r'\s', timex)[0]
			dmy = re.split(r'/|-', dmy)
			timex_val = str(dmy[2]) + '-' + str(dmy[1]) + '-' + str(dmy[0])

		# Specific dates
		elif re.match(r'\d{4}', timex):
			timex_val = str(timex)

		# Relative dates
		elif re.match(r'tonight|tonite|today', timex, re.IGNORECASE):
			timex_val = str(base_date)
		elif re.match(r'yesterday', timex, re.IGNORECASE):
			timex_val = str(base_date + RelativeDateTime(days=-1))
		elif re.match(r'tomorrow', timex, re.IGNORECASE):
			timex_val = str(base_date + RelativeDateTime(days=+1))

		# Weekday in the previous week.
		elif re.match(r'last ' + week_day, timex, re.IGNORECASE):
			day = hashweekdays[timex.split()[1]]
			timex_val = str(base_date + RelativeDateTime(weeks=-1, \
							weekday=(day,0)))

		# Weekday in the current week.
		elif re.match(r'this ' + week_day, timex, re.IGNORECASE):
			day = hashweekdays[timex.split()[1]]
			timex_val = str(base_date + RelativeDateTime(weeks=0, \
							weekday=(day,0)))

		# Weekday in the following week.
		elif re.match(r'next ' + week_day, timex, re.IGNORECASE):
			day = hashweekdays[timex.split()[1]]
			timex_val = str(base_date + RelativeDateTime(weeks=+1, \
							  weekday=(day,0)))

		# Last, this, next week.
		elif re.match(r'last week', timex, re.IGNORECASE):
			year = (base_date + RelativeDateTime(weeks=-1)).year

			# iso_week returns a triple (year, week, day) hence, retrieve
			# only week value.
			week = (base_date + RelativeDateTime(weeks=-1)).iso_week[1]
			timex_val = str(year) + 'W' + str(week)
		elif re.match(r'this week', timex, re.IGNORECASE):
			year = (base_date + RelativeDateTime(weeks=0)).year
			week = (base_date + RelativeDateTime(weeks=0)).iso_week[1]
			timex_val = str(year) + 'W' + str(week)
		elif re.match(r'next week', timex, re.IGNORECASE):
			year = (base_date + RelativeDateTime(weeks=+1)).year
			week = (base_date + RelativeDateTime(weeks=+1)).iso_week[1]
			timex_val = str(year) + 'W' + str(week)

		# Month in the previous year.
		elif re.match(r'last ' + month, timex, re.IGNORECASE):
			month = hashmonths[timex.split()[1]]
			timex_val = str(base_date.year - 1) + '-' + str(month)

		# Month in the current year.
		elif re.match(r'this ' + month, timex, re.IGNORECASE):
			month = hashmonths[timex.split()[1]]
			timex_val = str(base_date.year) + '-' + str(month)

		# Month in the following year.
		elif re.match(r'next ' + month, timex, re.IGNORECASE):
			month = hashmonths[timex.split()[1]]
			timex_val = str(base_date.year + 1) + '-' + str(month)
		elif re.match(r'last month', timex, re.IGNORECASE):

			# Handles the year boundary.
			if base_date.month == 1:
				timex_val = str(base_date.year - 1) + '-' + '12'
			else:
				timex_val = str(base_date.year) + '-' + str(base_date.month - 1)
		elif re.match(r'this month', timex, re.IGNORECASE):
				timex_val = str(base_date.year) + '-' + str(base_date.month)
		elif re.match(r'next month', timex, re.IGNORECASE):

			# Handles the year boundary.
			if base_date.month == 12:
				timex_val = str(base_date.year + 1) + '-' + '1'
			else:
				timex_val = str(base_date.year) + '-' + str(base_date.month + 1)
		elif re.match(r'last year', timex, re.IGNORECASE):
			timex_val = str(base_date.year - 1)
		elif re.match(r'this year', timex, re.IGNORECASE):
			timex_val = str(base_date.year)
		elif re.match(r'next year', timex, re.IGNORECASE):
			timex_val = str(base_date.year + 1)
		elif re.match(r'\d+ days? (ago|earlier|before)', timex, re.IGNORECASE):

			# Calculate the offset by taking '\d+' part from the timex.
			offset = int(re.split(r'\s', timex)[0])
			timex_val = str(base_date + RelativeDateTime(days=-offset))
		elif re.match(r'\d+ days? (later|after)', timex, re.IGNORECASE):
			offset = int(re.split(r'\s', timex)[0])
			timex_val = str(base_date + RelativeDateTime(days=+offset))
		elif re.match(r'\d+ weeks? (ago|earlier|before)', timex, re.IGNORECASE):
			offset = int(re.split(r'\s', timex)[0])
			year = (base_date + RelativeDateTime(weeks=-offset)).year
			week = (base_date + \
							RelativeDateTime(weeks=-offset)).iso_week[1]
			timex_val = str(year) + 'W' + str(week)
		elif re.match(r'\d+ weeks? (later|after)', timex, re.IGNORECASE):
			offset = int(re.split(r'\s', timex)[0])
			year = (base_date + RelativeDateTime(weeks=+offset)).year
			week = (base_date + RelativeDateTime(weeks=+offset)).iso_week[1]
			timex_val = str(year) + 'W' + str(week)
		elif re.match(r'\d+ months? (ago|earlier|before)', timex, re.IGNORECASE):
			extra = 0
			offset = int(re.split(r'\s', timex)[0])

			# Checks if subtracting the remainder of (offset / 12) to the base month
			# crosses the year boundary.
			if (base_date.month - offset % 12) < 1:
				extra = 1

			# Calculate new values for the year and the month.
			year = str(base_date.year - offset // 12 - extra)
			month = str((base_date.month - offset % 12) % 12)

			# Fix for the special case.
			if month == '0':
				month = '12'
			timex_val = year + '-' + month
		elif re.match(r'\d+ months? (later|after)', timex, re.IGNORECASE):
			extra = 0
			offset = int(re.split(r'\s', timex)[0])
			if (base_date.month + offset % 12) > 12:
				extra = 1
			year = str(base_date.year + offset // 12 + extra)
			month = str((base_date.month + offset % 12) % 12)
			if month == '0':
				month = '12'
			timex_val = year + '-' + month
		elif re.match(r'\d+ years? (ago|earlier|before)', timex, re.IGNORECASE):
			offset = int(re.split(r'\s', timex)[0])
			timex_val = str(base_date.year - offset)
		elif re.match(r'\d+ years? (later|after)', timex, re.IGNORECASE):
			offset = int(re.split(r'\s', timex)[0])
			timex_val = str(base_date.year + offset)

		# Remove 'time' from timex_val.
		# For example, If timex_val = 2000-02-20 12:23:34.45, then
		# timex_val = 2000-02-20
		timex_val = re.sub(r'\s.*', '', timex_val)

		# Substitute tag+timex in the text with grounded tag+timex.
		tagged_text = re.sub('<TIMEX2>' + timex_ori + '</TIMEX2>', '<TIMEX2 val=\"' \
			+ timex_val + '\">' + timex_ori + '</TIMEX2>', tagged_text)

	return tagged_text

####

def demo():
	import nltk
	text = nltk.corpus.abc.raw('rural.txt')[:10000]
	print TimeDetector.tag(text)

if __name__ == '__main__':
	demo()



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path
import os

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
	long_description = f.read()

setup(
	name='dragonfire',

	# Versions should comply with PEP440.  For a discussion on single-sourcing
	# the version across setup.py and the project code, see
	# https://packaging.python.org/en/latest/single_source_version.html
	version='0.8.6',

	description='Dragonfire is an open source virtual assistant project for Ubuntu based Linux distributions',
	long_description=long_description,

	# The project's main homepage.
	url='https://github.com/mertyildiran/Dragonfire',

	# Author details
	author='Mehmet Mert Yildiran',
	author_email='mert.yildiran@bil.omu.edu.tr',

	# Choose your license
	license='MIT',

	# See https://pypi.python.org/pypi?%3Aaction=list_classifiers
	classifiers=[
		# How mature is this project? Common values are
		#   3 - Alpha
		#   4 - Beta
		#   5 - Production/Stable
		'Development Status :: 3 - Alpha',

		# Indicate who your project is intended for
		'Intended Audience :: Developers',
		'Topic :: Scientific/Engineering :: Human Machine Interfaces',

		# Pick your license as you wish (should match "license" above)
		'License :: OSI Approved :: MIT License',

		# Intended language
		'Natural Language :: English',

		# Target Operating System
		'Operating System :: POSIX :: Linux',

		# Specify the Python versions you support here. In particular, ensure
		# that you indicate whether you support Python 2, Python 3 or both.
		'Programming Language :: Python :: 2.7',
	],

	# What does your project relate to?
	keywords='virtual assistant machine learining artifical intelligence chat bot',

	# You can just specify the packages manually here if your project is
	# simple. Or you can use find_packages().
	packages=find_packages(),

	# Alternatively, if you want to distribute just a my_module.py, uncomment
	# this:
	#   py_modules=["my_module"],

	# List run-time dependencies here.  These will be installed by pip when
	# your project is installed. For an analysis of "install_requires" vs pip's
	# requirements files see:
	# https://packaging.python.org/en/latest/requirements.html
	install_requires=['wikipedia','PyUserInput','google-api-python-client','SpeechRecognition','nltk','egenix-mx-base','httplib2>=0.9.1'],

	# List additional groups of dependencies here (e.g. development
	# dependencies). You can install these using the following syntax,
	# for example:
	# $ pip install -e .[dev,test]
	extras_require={},

	# If there are data files included in your packages that need to be
	# installed, specify them here.  If using Python 2.6 or less, then these
	# have to be included in MANIFEST.in as well.
	package_data={
		# If any package contains *.txt files, include them:
		'': ['*.txt'],
		'': ['*.ini'],
		'': ['*.jconf'],
		'': ['*.dfa'],
		'': ['*.dict'],
		'': ['*.grammar'],
		'': ['*.term'],
		'': ['*.voca'],
		'': ['*.aiml'],
	},

	# Although 'package_data' is the preferred approach, in some case you may
	# need to place data files outside of your packages. See:
	# http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
	# In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
	data_files=[],

	# To provide executable scripts, use entry points in preference to the
	# "scripts" keyword. Entry points provide cross-platform support and allow
	# pip to create the appropriate form of executable for the target platform.
	entry_points={
		'console_scripts': [
			'dragonfire=dragonfire:initiate',
		],
	},
)



import json
import urllib
import urllib2
import time
import sys
import contextlib
import cStringIO
from random import randint

class YodaQA():

	@staticmethod
	def answer(address, query, user_prefix):
		url = address + '/q'
		#url = 'http://localhost:4567/q'
		post_params = {
			'text' : query
		}

		params = urllib.urlencode(post_params)
		response = urllib2.urlopen(url, params)
		json_response = json.loads(response.read())

		#print json_response['id']

		# setup toolbar
		toolbar_width = 180 / 3
		sys.stdout.write("|%s|" % (" " * toolbar_width))
		sys.stdout.flush()
		sys.stdout.write("\b" * (toolbar_width+1)) # return to start of line, after '['

		seconds = 0
		while seconds < 180:
			time.sleep(3)
			seconds = seconds + 3

			url2 = url + '/' + json_response['id']

			#print url2
			response2 = urllib2.urlopen(url2)
			data = json.load(response2)
			#print data
			#print data['answers']
			sys.stdout.write(unichr(0x2588))
			sys.stdout.flush()
			if data['answers']:
				if float(data['answers'][0]['confidence']) > 0.5:
					sys.stdout.write("\n\n")
					return data['answers'][0]['text']
		sys.stdout.write("\n\n")
		return noanswer(user_prefix)




def noanswer(user_prefix):
	words_dragonfire = {
	0 : "I'm not that smart, " + user_prefix + ".",
	1 : "Please, be simple.",
	2 : "Excuse me? I have an average IQ."
	}
	return words_dragonfire[randint(0,2)]

@contextlib.contextmanager
def nostdout():
	save_stdout = sys.stdout
	sys.stdout = cStringIO.StringIO()
	yield
	sys.stdout = save_stdout

@contextlib.contextmanager
def nostderr():
	save_stderr = sys.stderr
	sys.stderr = cStringIO.StringIO()
	yield
	sys.stderr = save_stderr

if __name__ == "__main__":
	print YodaQA.answer("http://localhost:4567", "When was Albert Einstein born", "sir")
